
#include <Stepper.h>

const int stepsPerRevolution = 5;  	// change this to fit the number of steps per revolution
									// for your motor

// initialize the stepper library on pins 3 through 6:
Stepper leftStepper(stepsPerRevolution, 3, 4);
Stepper rightStepper(stepsPerRevolution, 5, 6);

//Function for moving forword by any distance 
//using the stepper distance times
void forword(int distance) {
	for (int i = 0; i < distance; ++i)
	{
		leftStepper.step(-stepsPerRevolution);
  		rightStepper.step(stepsPerRevolution);
	}
		
}

//Function for rotating left by any distance 
//using the stepper distance times
void left(int distance) {
  	for (int i = 0; i < distance; ++i)
  	{
  		rightStepper.step(-stepsPerRevolution);	
        leftStepper.step(-stepsPerRevolution);
  	}
  	
}


//Function for rotating right left by any distance 
//using the stepper distance times
void right(int distance) {
  	for (int i = 0; i < distance; ++i)
  	{
  		leftStepper.step(stepsPerRevolution);	
        rightStepper.step(stepsPerRevolution);   
  	}
  	
}

//Function for moving backwards by any distance
//using the stepper distance times
void backwards(int distance) {
	for (int i = 0; i < distance; ++i)
	{
		leftStepper.step(stepsPerRevolution);
  		rightStepper.step(-stepsPerRevolution);
	}
		
}





//calculates the average value of 3 input pins over a given amount of time (timeX)
//values below the cutoff value are counted as zero
//function can be used to determine strenght of audio signal
long* calculate_average (int pin_number,int pin_number2,int pin_number3,int timeX,int cutoff,int cutoff2,int cutoff3) {
    
	//initialize sum to sum up values
    long sum = 0;
    long sum2 = 0;
    long sum3 = 0;

    //measure timeX times
    for (int i = 0; i < timeX; ++i)
    {
    	//save the measured data
      	long value  = analogRead(pin_number);
      	long value2 = analogRead(pin_number2);    
      	long value3 = analogRead(pin_number3);    
      	
      	//check for each value if is higher than the cutoff frequency
    	if (value >= cutoff)
    	{
      		sum  += value;
    	}
    	if (value2 >= cutoff2)
    	{
    	  	sum2 += value2;
    	}
    	if (value3 >= cutoff3)
    	{
    		sum3 += value3;
    	}
          
    }
    //allocate memory for the return value array
    long* return_value = malloc(sizeof(long)*3);

    //assign return values
    return_value[0] = (sum/timeX);
    return_value[1] = (sum2/timeX);
    return_value[2] = (sum3/timeX);
    
    //return array
    return return_value;
}


//compares two values given a tolerance

long compare (long value_long,long value2_long,int tolerance) {
	//cast values to integer because long does not support negative numbers
	int value = (int) value_long;
	int value2 = (int) value2_long;

	//if value one is negative return 2
	if (value < 0)
	{
		return 2;
	}
	//if value 2 is negative return 1 
	if (value2 < 0)
	{
		return 1;
	}
	//if value1 one is tolerance greater than value2 return 1
	if (value >= value2)
    {
        if (value-tolerance > value2)
        {
            return 1;
        }
        
    }
    //if value2 one is tolerance greater than value1 return 2
    else if (value2 >= value) 
    {
        if (value2-tolerance > value)
        {
            return 2;
        }
        
    }
    //otherwise return 0 
    //(both positive and value and value 2 are similar enough regarding the tolerance)
    return 0;
}

void setup() {
  // set the speed for the stepper
  leftStepper.setSpeed(7000);
  rightStepper.setSpeed(7000);

  //Serial.begin(9600);
}

void loop() {


	//calculate the average value on each microphone (19,20,21, over 1000 measurements)
    long* average =  calculate_average(19,20,21,1000,535,535,520);     //18:A4 19:A5
    
    //cast the value from the front microphone to Integer because we need to subtract
    int front_mic = (int) average[2];
    //subtract 400 from that value because that microphone is different 
    //(would not be needed if all microphones are equal)
    front_mic -= 400;

    //compare left and right microphone
    long maxvalue = compare(average[0],average[1],75);
    //if left microphone has a stronger signal move left
    if (maxvalue == 1)
    {
    	left(10);
    }
    //if right microphone has a stronger signal move right
    if (maxvalue == 2)
    {
    	right(10);
    }
    //if both are equal regarding the tolerance  
    if (maxvalue == 0)
    {
    	//compare right and front with smaller tolerance than maxvalue
    	long compare_right = compare(front_mic,average[1],50);
    	//compare left and front with smaller tolerance than maxvalue
    	long compare_left = compare(front_mic,average[0],50);
    	//compare left and right microphone with tolerance of 0
    	long compare_new = compare(average[0],average[1],0);

    	//if right microphone has a stronger signal then the front one and is actually greater than the left one rotate right
    	if ((compare_right == 2 || compare_right == 0) && compare_new == 2)
    	{
    		right(10);
    	}
    	//if left microphone has a stronger signal then the front one and is actually greater than the right one rotate left
    	else if((compare_left == 2 || compare_left == 0) && compare_new == 1) 
    	{
    		left(10);
    	}
    	//otherwise move forword since the signal comes from up front
    	else 
    	{
    		forword(10);
    	}

    }

    //free average
    free(average);
  
}

